@@ -51,11 +51,11 @@ Html form rendering example with default values
 
 Full tutorial on yattag.org_
 
-.. _yattag.org: http://www.yattag.org
+.. _yattag.org: https://www.yattag.org
 """
 
 __author__ = "Benjamin Le Forestier (benjamin@leforestier.org)"
-__version__ = '1.11.2'
+__version__ = '1.12.0'
 
 from yattag.simpledoc import SimpleDoc
 from yattag.doc import Doc
